/**
 * Interface definition for a VoterManager
 * Manages a bunch of voters that can be created
 * from a remote client
 * Responsible for instantiation & termination
 * of the Voters
 * @Author Preethi Vishwanath
 * @Date 03/03/07
 */

import java.util.*;

interface VoterManager extends java.rmi.Remote 
{	
	/*
	 * First method to setup a voter on a machine
	 * @parameters
	 * Delay: Defines the delay between successive
	 * 	  actions for the voter
	 * Faulty: Is this voter a faulty voter, faulty
	 *	   voters cannot be relied upon
	 * vID: An identity assigned to the voter
	 * mID: Logical identity of the machine on which the
	 *	voter resides
	 * ht_blobMachine: Hashtable contains as keys the blob-ids
	 *		   that this voter is interested in.
	 *		   On termination, this will also contain
	 *		   the final blob->Machine mapping
	 * blob_mcID: 	   Initial blob->machine mapping.
	 *		   Necessary for allowing each voter to access
	 *	 	   its blob
	 * mcID_IP:	   MachineID->IP Address mapping.
	 * db, user, passwd, tblName: Table config parameters (mostly, unused)
         *
	 * @returns:A reference to the remote Voter.
	 */


	public Voter createVoter(int delay, 
				 boolean flty,
				 int vID,
				 int mcID,
				 Hashtable ht_blobMachine,
				 int[] blob_mcID,
				 String[] mcID_tIP,
				 String db, 
				 String user, 
				 String passwd, 
				 String tblName) 
	throws java.rmi.RemoteException, 
	       ClassNotFoundException, 
	       java.sql.SQLException;

}
