#!/usr/bin/perl -w
use strict;

my $line = "";
my @line_arr = ();
my @blob_arr = ();

my %blob_o_loc;
my %blob_n_loc;

my %blob_o_freq;
my %blob_n_freq;

my %blob_voter;

my $e_VOTER = 0;
my $e_VOTERMC = 1;
my $e_BLOBID = 2;
my $e_BLOBOLDMC = 3;
my $e_BLOBOLDMCDIST = 4;
my $e_BLOBNEWMC = 5;
my $e_BLOBNEWMCDIST = 6;
my $e_BLOBFREQ = 7;
my $e_BLOBOLDSCALE = 8;
my $e_BLOBNEWSCALE = 9;

my $OLD_BLOB_CUTOFF = 0;
my $NEW_BLOB_CUTOFF = 0;

my $NUM_DECAYED_SAMPLES = 10;

my $idx = 0;
my $b_idx = 0;
my $blob_id;

while ($line = <>) {
	chomp($line);
	next if (!($line =~ qr/STAT_NEW_FREQ\s*:(.*)/));
	@line_arr = split(/[:]/, $1);
	$b_idx = 0;
	for ($idx = 0; $idx <= $#line_arr; $idx ++) {
		next if (!($idx%2));
		$blob_arr[$b_idx] = $line_arr[$idx];
		$b_idx ++;
	}
	$blob_id = $blob_arr[$e_BLOBID];
	
	# Compute old frequency of access for every blob
	if (!exists ($blob_o_freq{$blob_id})) {
		$blob_o_freq{$blob_id} = $blob_arr[$e_BLOBOLDSCALE];
	} else {
		$blob_o_freq{$blob_id} += $blob_arr[$e_BLOBOLDSCALE];
	}
	# Compute new frequency of access for every blob
	if (!exists($blob_n_freq{$blob_id})) {
		$blob_n_freq{$blob_id} = $blob_arr[$e_BLOBNEWSCALE];
	} else {
		$blob_n_freq{$blob_id} += $blob_arr[$e_BLOBNEWSCALE];
	}
	# Obtain the old blob->location mapping
	
	if (!exists($blob_o_loc{$blob_id})) {
		$blob_o_loc{$blob_id} = $blob_arr[$e_BLOBOLDMC];
	}
	# Obtain the new blob->location mapping
	if (!exists($blob_n_loc{$blob_id})) {
		$blob_n_loc{$blob_id} = $blob_arr[$e_BLOBNEWMC];
	}
	

	# The first element is the count for OLD SCALE
	# The second element is the count for the  NEW SCALE

	if (!exists ($blob_voter{$blob_id})) {
		$blob_voter{$blob_id} = [[$blob_arr[$e_VOTER],
					    
					 0, 0,  
					 $blob_arr[$e_BLOBOLDSCALE], 
					 $blob_arr[$e_BLOBNEWSCALE],
					 $blob_arr[$e_VOTERMC]
					 ]]; 
	} else {
		my $arr = $blob_voter{$blob_id};
		if (!update_voter($arr, \@blob_arr)) {
			push @$arr, [$blob_arr[$e_VOTER],
				      0, 0, 
				     $blob_arr[$e_BLOBOLDSCALE], 
				     $blob_arr[$e_BLOBNEWSCALE],
				     $blob_arr[$e_VOTERMC]
				     ];
			$blob_voter{$blob_id} = $arr;
		} 	
	}				
}



my $key;
my $value;

#print STDOUT "BLOB #, FREQUENCY(OLD)\n";
#foreach $key (sort { $blob_o_freq{$b} <=> $blob_o_freq{$a} } keys %blob_o_freq) {
#	$value = $blob_o_freq{$key};
#	print STDOUT "$key, $value\n";
#}

# THIS IS NOT NECESSARY
#print STDOUT "VOTER #, BLOB \n";
#foreach $key (sort sort_by_voter keys %blob_voter) {
#	$value = $blob_voter{$key};
#	for ($idx = 0; $idx <= $#$value; $idx ++) {
#		print STDOUT "$value->[$idx]->[0], $key\n";
#	}	
#}


# First update for BLOBS that have 30% or greater access

my $total_axs_old = 0;
my $total_axs_new = 0;
my %per_blob_axs_old;
my %per_blob_axs_new;

foreach $key (keys %blob_voter) {
	$value = $blob_voter{$key};
	if (!exists $per_blob_axs_old{$key}) {
		$per_blob_axs_old{$key} = 0;
	}
	if (!exists $per_blob_axs_new{$key}) {
		$per_blob_axs_new{$key} = 0;
	}
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		$total_axs_old += $value->[$idx]->[3];
		$per_blob_axs_old{$key} += $value->[$idx]->[3];
		$total_axs_new += $value->[$idx]->[4];
		$per_blob_axs_new{$key} += $value->[$idx]->[4];
	}
	
}

foreach $key (keys %blob_voter) {
	$value = $blob_voter{$key};

	# Update the fractional accesses to this Blob
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		$value->[$idx]->[1] = ($per_blob_axs_old{$key}/$total_axs_old)*100;
		$value->[$idx]->[2] = ($per_blob_axs_new{$key}/$total_axs_new)*100;
	}	


}


#print STDOUT "BLOB#, FRAC_FREQ_OLD(MIN: $OLD_BLOB_CUTOFF), VOTER#, OLD_FREQ\n";
foreach $key (sort sort_by_old keys(%blob_voter)) {
	$value = $blob_voter{$key};
	my $frac_freq_old = $value->[0]->[1];
	next if ($frac_freq_old < $OLD_BLOB_CUTOFF);
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		my $voter_id = $value->[$idx]->[0];
		my $frac_freq_old = $value->[$idx]->[1];
		my $old_scale = $value->[$idx]->[3];
#		print STDOUT "$key, $frac_freq_old, $voter_id, ";
#		print STDOUT "$old_scale\n";
	}

}

# Next you have to plot the decay of old frequencies of access of various
# BLOBS by different voters as a function of time
#print STDOUT "BLOB#, VOTER#, INITIAL_VALUE, ";
for ($idx = 0; $idx < $NUM_DECAYED_SAMPLES; $idx ++) {
#	print STDOUT "VALUE_$idx, ";	
}
#print STDOUT "\n";

my $f_idx = 0;
$b_idx = 0;
my $curr_decay = 0;
my $curr_val = 0;
foreach $key (sort sort_by_blob_voter keys(%blob_voter)) {
	$value = $blob_voter{$key};
	$b_idx ++;
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		my $voter_id = $value->[$idx]->[0];
		my $old_scale = $value->[$idx]->[3];
	#	print STDOUT "$key, $voter_id, $old_scale ";
		for ($f_idx = 0; $f_idx <= $NUM_DECAYED_SAMPLES; $f_idx ++) {
			$curr_decay = $f_idx * $b_idx;
			$curr_val = $old_scale*(1 - exp(-$curr_decay));
			$curr_val = sprintf "%.2f", $curr_val; 
	#		print STDOUT ", $curr_val";
		}			
	#	print STDOUT "\n";
	}
	
}

#print STDOUT "BLOB #, FREQUENCY(NEW)\n";
#foreach $key (sort { $blob_n_freq{$b} <=> $blob_n_freq{$a} } keys %blob_n_freq) {
#	$value = $blob_n_freq{$key};
#	print STDOUT "$key, $value\n";
#}

#print STDOUT "BLOB#, FRAC_FREQ_NEW(MIN: $NEW_BLOB_CUTOFF), VOTER#, NEW_FREQ\n";

foreach $key (sort sort_by_new keys(%blob_voter)) {
	$value = $blob_voter{$key};
	my $frac_freq_new = $value->[0]->[2];
	next if ($frac_freq_new < $NEW_BLOB_CUTOFF);
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		my $voter_id = $value->[$idx]->[0];
		my $new_scale = $value->[$idx]->[4];
#		print STDOUT "$key, $frac_freq_new, $voter_id, ";
#		print STDOUT "$new_scale\n";
	}

}

print STDOUT "BLOB#, OLD_LOC, NEW_LOC\n";
foreach $key (sort { $a <=> $b } keys %blob_o_loc) {
	print STDOUT "$key, $blob_o_loc{$key}, $blob_n_loc{$key}\n";
}

print STDOUT "BLOB#, CUM_OLD_BW, CUM_NEW_BW\n";
foreach $key (sort { $a <=> $b } keys %blob_o_freq) {
	print STDOUT "$key, $blob_o_freq{$key}, $blob_n_freq{$key}\n";
}

print STDOUT "BLOB#, VOTER#, VOTER_LOC, OLD_BW, NEW_BW\n";
foreach $key(sort { $a <=> $b } keys %blob_voter) {
	$value = $blob_voter{$key};
	for ($idx = 0; $idx <= $#$value; $idx ++) {
		print STDOUT "$key, ";
		print STDOUT "$value->[$idx]->[0], ";
		print STDOUT "$value->[$idx]->[5], ";
		print STDOUT "$value->[$idx]->[3], ";
		print STDOUT "$value->[$idx]->[4]";
		print STDOUT "\n";
	}

}



sub sort_by_old
{
	$blob_voter{$b}->[0]->[1] <=> $blob_voter{$a}->[0]->[1]

}

sub sort_by_new
{
	$blob_voter{$b}->[0]->[2] <=> $blob_voter{$a}->[0]->[2]

}

sub sort_by_blob_voter
{
	($blob_voter{$b}->[0]->[1] <=> $blob_voter{$a}->[0]->[1])
		    ||
	($blob_voter{$a}->[0]->[0] <=> $blob_voter{$b}->[0]->[0])

}

sub sort_by_voter
{
	
	$blob_voter{$a}->[0]->[0] <=> $blob_voter{$b}->[0]->[0]


}

sub update_voter
{

	my $arr_ref = shift;
	my $blob_ref = shift;

	my $arr_idx = 0;
	for ($arr_idx = 0; $arr_idx <= $#$arr_ref; $arr_idx ++) {
		if ($arr_ref->[$arr_idx]->[0] == $blob_ref->[$e_VOTER]) {
			$arr_ref->[$arr_idx]->[1] += $blob_ref->[$e_BLOBOLDSCALE];
			$arr_ref->[$arr_idx]->[2] += $blob_ref->[$e_BLOBNEWSCALE];
			return 1;
		}

	}	
	return 0;

}
