
#include <stdio.h>
#include "xmlparse.h"
#include "xml_node.h"
#include <string.h>
#include <ctype.h>

struct xml_node* curr_node = NULL;
struct xml_node* head_node = NULL;
struct addr_rec* rec_list = NULL;
struct addr_rec* head_rec_list = NULL;

#define MAX_STR_LEN 100

void delete_xml_tree(struct xml_node* head_node);
void delete_addr_rec(struct addr_rec* rec_list);

//Uncomment the if 0 below & the corresponding #elsif for
// testing the XML driver parser.

//#if 0 
#include <postgres.h>
#include <string.h>
#include <fmgr.h>
#include <funcapi.h>
PG_FUNCTION_INFO_V1(get_xml_data);
PG_FUNCTION_INFO_V1(str_example);


Datum
get_xml_data(PG_FUNCTION_ARGS)
{
	FuncCallContext *f_ctx;
	MemoryContext oldC;

	TupleDesc tup_desc;
	AttInMetadata *attinmeta;
	int call_cntr;
	int max_calls;
	struct addr_rec* xml_addr = NULL;
	text* f_name;
	int32 fsize;	

	if (SRF_IS_FIRSTCALL()) {
		f_ctx = SRF_FIRSTCALL_INIT();
		oldC = MemoryContextSwitchTo(f_ctx->multi_call_memory_ctx);
		f_name = PG_GETARG_TEXT_P(0);
		f_ctx->max_calls = PG_GETARG_UINT32(1);
		fsize = VARSIZE(f_name) - VARHDRSZ;
		create_xml_tree((char *)VARDATA(f_name), fsize);
		if (get_call_result_type(fcinfo, NULL, 
					 &tup_desc) != TYPEFUNC_COMPOSITE) {
			ereport(ERROR,
				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
				 errmsg("function returning record called"
					"in contect that cannot accept"
					"type record")));	
		}
		attinmeta = TupleDescGetAttInMetadata(tup_desc);
		f_ctx->attinmeta = attinmeta;
		f_ctx->user_fctx = (void *)head_rec_list;				
		MemoryContextSwitchTo(oldC);
	}
	f_ctx = SRF_PERCALL_SETUP();
	call_cntr = f_ctx->call_cntr;
	max_calls = f_ctx->max_calls;
	attinmeta = f_ctx->attinmeta;
	xml_addr = (struct addr_rec *)f_ctx->user_fctx;
	if (call_cntr < max_calls) {
		char **values;
		HeapTuple tuple;
		Datum result;
		
		values = (char **)palloc(2* sizeof(char *));
		values[0] = (char *)palloc(MAX_STR_LEN * sizeof(char));
		values[1] = (char *)palloc(MAX_STR_LEN * sizeof(char));
		
		snprintf(values[0], MAX_STR_LEN, xml_addr->voter_id);
		snprintf(values[1], MAX_STR_LEN, xml_addr->blob_id);

		tuple = BuildTupleFromCStrings(attinmeta, values);
		result = HeapTupleGetDatum(tuple);
		
		pfree(values[0]);
		pfree(values[1]);

		pfree(values);
		f_ctx->user_fctx = (void *)(xml_addr->next);
		SRF_RETURN_NEXT(f_ctx, result);	
		
	} else {
		//delete_xml_tree(head_node);
		//delete_addr_rec(head_rec_list);	
		head_node = curr_node = NULL;
		head_rec_list = rec_list = NULL;	
		SRF_RETURN_DONE(f_ctx);
	}


}
//#endif

void delete_xml_tree(struct xml_node* head_node)
{
	unsigned int idx = 0;
	if (head_node == NULL) return;
	if (head_node->nchilds == 0) {
		if (head_node->name != NULL) {
			free(head_node->name);
			head_node->name = NULL;
		}
		if (head_node->value != NULL) {
			free(head_node->value);
			head_node->value = NULL;
		}
		free(head_node);
		return;		
	}
	while (head_node->child[idx] != NULL) {
		delete_xml_tree(head_node->child[idx]);
		head_node->nchilds --;
		idx ++;
	}
	


}
void delete_addr_rec(struct addr_rec* rec_list)
{
	struct addr_rec* next_node = NULL;
	if (rec_list == NULL) return;
	next_node = rec_list->next;
	free(rec_list);
	rec_list = NULL;
	delete_addr_rec(next_node);
	

}

int is_empty(const char* str, int len)
{
	unsigned int idx = 0;
	while (idx < len) {
		if (!isspace(str[idx])) return 0;
		idx ++;
	}
	return 1;
}

void charHandle(void *userData,
		const XML_Char *s,
		int len)
{
	char* my_val;
	if (is_empty((const char *)s,len)) return; 
	my_val = (char *)malloc((len+1)*sizeof(char));
	memcpy(my_val, s, len);
	my_val[len] = '\0';	
	curr_node->value = my_val;
}

struct xml_node* create_node(const char* name, int depth)
{
	unsigned int idx = 0;
	struct xml_node* new_node;
	char* my_nm = (char *)malloc((strlen(name) + 1)*sizeof(char));
	strncpy(my_nm, name, strlen(name) + 1);
	my_nm[strlen(name)] = '\0';
	
	new_node = (struct xml_node *)malloc(sizeof(struct xml_node ));
	memset(new_node, 0, sizeof(struct xml_node));
	new_node->depth = depth;
	new_node->prev = NULL;
	for (idx = 0; idx < MAX_CHILD ; idx ++) {
		new_node->child[idx] = NULL;
	}
	new_node->name = my_nm;
	new_node->nchilds = 0;
	return new_node;

}

void startElement(void *userData, const char *name, const char **atts)
{
  struct xml_node* tmp_node = NULL;
  if (curr_node == NULL) {
	curr_node = create_node(name, 0);
	head_node = curr_node;
  } else {
	tmp_node = create_node(name, (curr_node->depth + 1));
	curr_node->child[curr_node->nchilds] = tmp_node;
	curr_node->nchilds ++;
	tmp_node->prev = curr_node;
	curr_node = tmp_node;
   }
   
}

struct addr_rec* create_addr_rec(void)
{
	struct addr_rec* tmp = NULL;
	tmp = (struct addr_rec *)malloc(sizeof(struct addr_rec));
	tmp->voter_id = NULL;
	tmp->blob_id = NULL;
	tmp->next = NULL;
	return tmp;

}

void endElement(void *userData, const char *name)
{
	//printf("DEPTH[%u]:ELEM:[%s]=%s\n", curr_node->depth, curr_node->name, curr_node->value);
	if (rec_list == NULL) {
		rec_list = create_addr_rec();
		head_rec_list = rec_list;
	}
	if (curr_node->nchilds == 0) {
		char* val = curr_node->value;
		if (rec_list->voter_id == NULL) {
			rec_list->voter_id = val;
		} else if (rec_list->blob_id == NULL) {
			rec_list->blob_id = val;			
		} else if (rec_list->next == NULL) {
			rec_list->next = create_addr_rec();
			rec_list = rec_list->next;
			rec_list->voter_id = val;
		}
	}
	curr_node = curr_node->prev;
}

void print_xml_tree(struct xml_node* head_node)
{
	unsigned int idx = 0;
	if (head_node == NULL) 	return;
	if (head_node->name != NULL) {
		printf("DEPTH = %u NAME = %s", head_node->depth, head_node->name);
	}
	if (head_node->value != NULL) {
		printf("VALUE = %s\n", head_node->value);
	}
	if (head_node->nchilds == 0) {
		return;
	}
	printf("\n");
	fflush(stdout);		
	for (idx = 0; idx < head_node->nchilds; idx ++) {
		print_xml_tree(head_node->child[idx]);
	}

}
void print_addr_rec(struct addr_rec* ll_node)
{
	if (ll_node == NULL) return;
	printf("Voter Id : %s: "
		"BLOB Id: %s\n", 
		ll_node->voter_id,
		ll_node->blob_id
		);
	fflush(stdout);
	print_addr_rec(ll_node->next);
}

void create_xml_tree(char* fname, int size)
{
  char buf[BUFSIZ];
  int done;
  XML_Parser parser = XML_ParserCreate(NULL);
 
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser, charHandle);
  fname[size] = '\0';
  FILE *fp = fopen(fname, "r");
  do {
    size_t len = fread(buf, 1, sizeof(buf), fp);
    done = len < sizeof(buf);
    if (!XML_Parse(parser, buf, len, done)) {
      fprintf(stderr,
	      "%s at line %d\n",
	      XML_ErrorString(XML_GetErrorCode(parser)),
	      XML_GetCurrentLineNumber(parser));
      return;
    }
  } while (!done);
  
  fclose(fp);

  XML_ParserFree(parser);
  
  return;
}
