#!/bin/sh

# Run this script as root on both machines.
# You need to have all files under the same directory
# structure for both client and server.

# These are the only two variables you need to change
# in this script.
# You need to run this script first on the manager

MANAGER=192.168.0.103
CLIENT=192.168.0.103


# You should have to change these

PREFIX=192.168
C_USER=hlthantr

# These are purely derived, DO NOT TOUCH!
do_mgr=0

function clean_db {
	make -f Makefile clean
}
function setup_db {
	make -f Makefile
	psql $1 $2

}

function stop_service {
	s_name=$1
	nlines=`ps -eaf | grep $s_name | wc -l`
	pid=`ps -eaf | grep $s_name | grep -v 'grep' | awk '{print $2}'`
	if [ "${nlines}" = "2" ]; then
		echo "STOPPING SERVICE $sname: PID: $pid"
		kill $pid
	elif [ "$pid" != "" ]; then
		kill $pid
	fi	


}

function cleanup {

	echo "Cleaning up prior installation.....";	
	# Create the HTML directory if didn't exist on the webserver
	if [ ! -d /var/www/html/proj6 ]; then
		mkdir /var/www/html/proj6
	fi
	rm -f /var/www/html/proj6/*.class

	# Remove all the skeletal files
	rm -f *.class
	rm -f VoterManagerImpl_S*.java
	rm -f VoterByzantineImpl_S*.java

	# Check your identity
	ipaddr=`ifconfig | grep $PREFIX | cut -d':' -f2 | cut -d' ' -f1`

	if [ "$ipaddr" = "$CLIENT" ]; then
		#peer_ip=192.168.0.107
		peer_ip=$MANAGER
		do_mgr=0
	else
		#peer_ip=192.168.0.102
		peer_ip=$CLIENT
		do_mgr=1
	fi
	echo "....Done"
	echo "Killing daemon RMI services....."
	stop_service VoterServer
	stop_service rmiregistry
	stop_service Manager
	psql testdb testuser -c 'DROP TABLE tblMachByzantine'
	echo "....Done"

}
function compile {
	echo "Compiling project...."
	cd /home/$C_USER/proj6

	# First compile all the java classes
	javac *.java

	# Run rmic on all the implementation classes
	rmic VoterManagerImpl
	rmic VoterByzantineImpl
	
	# Copy all the files into the HTTP server
	cp /home/$C_USER/proj6/*.class /var/www/html/proj6
	echo "...Done"	
}
if [ "$USER" != "root" ]; then
	echo "You must be root to execute this script";
	exit
fi

# Capture output of everything you do to a file
#
script_file=`date +%m%d_%H%M`
script_file=log_${script_file}

service postgresql restart
clean_db
setup_db testdb testuser
cleanup
compile
echo "Starting RMI registry...."
rmiregistry & 
echo "....Done"
echo "Setting CLASSPATH..."
export CLASSPATH=`pwd`:/var/www/html/proj6:/usr/share/java/postgresql-jdbc3.jar:.
echo "...Done"
echo "Running VoterServer....";
java -Djava.rmi.server.codebase=http://${ipaddr}/proj6 -Djava.rmi.server.hostname=${ipaddr} -Djava.security.policy=rmi.server.policy VoterServer &

sleep 2 
echo "....Done";
if test ${do_mgr} = 0; then
	echo "CLIENT: running the Byzantine Manager...."
	sleep 2
	java -Djava.rmi.server.codebase=http://${ipaddr}/proj6 -Djava.security.policy=rmi.client.policy Manager
	echo "....Done"
fi


